// AgentU000000X.java
// CS3243 Homework 1: Lines of Action

// Lines of Action agent.

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.List;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;

public class AgentU000000X {
	// Agent identity and email.
	private static final String IDENTITY = "The NuaH agent: a lazy player that makes random moves";
	private static final String EMAIL = "u000000@nus.edu.sg";

	// The maximum amount of time the agent is allowed to execute, and the
	// default move when the agent is out of time.
	private static final long URGENT_TIME = 2800;
	private static final String URGENT_MOVE = "A1B2";

	// The color this agent is playing as.
	private static final LOAColor MY_COLOR = LOAColor.BLACK;

	// Class that implements a pseudo timer interrupt. The run() method will
	// be run after MAX_EXECUTION_TIME milliseconds.
	private static class UrgentMoveTask extends TimerTask {
		// The run() method of the TimerTask class. Do not think, just
		// pass and exit.
		public void run() {
			System.out.println(URGENT_MOVE);
			System.exit(0);
		}
	}

	// Method that displays an error message and exits.
	private static void error(String message) {
		System.out.println(message);
		System.exit(1);
	}

	// Agent move function. 
	private static LOAMove makeMove(LOABoard board, int turn) {
		List<LOAMove> legalMoveList = board.getLegalMoves(MY_COLOR);
		LOAMove[] legalMoves = new LOAMove[legalMoveList.size()];
		legalMoveList.toArray(legalMoves);
		if (legalMoves.length == 0)
			error("No legal moves!");
		Random random = new Random();
		int choice = random.nextInt(legalMoves.length);
		return legalMoves[choice];
	}

	// Reads in the turn number.
	private static int readTurn(BufferedReader reader) throws IOException {
		String line = reader.readLine();
		if (line == null)
			error("Missing turn number!");
		line = line.trim();
		int turn = 0;
		try {
			turn = Integer.parseInt(line);
		}
		catch (NumberFormatException e) {
			error("Invalid turn number!");
		}
		return turn;
	}

	// Reads in the board.
	private static LOABoard readBoard(BufferedReader reader) throws IOException {
		String line = reader.readLine();
		if (line == null)
			error("Missing board size!");
		line = line.trim();
		int size = 0;
		try {
			size = Integer.parseInt(line);
		}
		catch (NumberFormatException e) {
			error("Invalid board size!");
		}
		String input = "";
		for (int i = 0; i < size; i++) {
			line = reader.readLine();
			if (line == null)
				error("Invalid board!");
			input = input + line.trim();
		}
		LOABoard board = null;
		try {
			board = new LOABoard(size, input);
		}
		catch (IllegalArgumentException e) {
			error("Invalid board!");
		}
		return board;
	}

	// Main method.
	public static void main(String[] args) throws IOException {
		BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
		String line = reader.readLine();
		if (line == null)
			error("Missing command!");
		line = line.trim();
		if (line.equals("identify"))
			System.out.println(IDENTITY);
		else if (line.equals("move")) {
			Timer timer = new Timer();
			timer.schedule(new UrgentMoveTask(), URGENT_TIME);
			int turn = readTurn(reader);
			LOABoard board = readBoard(reader);
			LOAMove move = makeMove(board, turn);
			timer.cancel();
			System.out.println(move);
		}
		else {
			System.out.println("Unknown command!");
			System.exit(1);
		}
	}
}

