// LOAMove.java
// CS3243 Homework 1: Lines of Action

// Class for representing a move.

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class LOAMove implements Comparable<LOAMove> {
	// Private class members.
	private int startRow;
	private int startCol;
	private int endRow;
	private int endCol;

	// Constructor from coordinates.
	public LOAMove(int startRow, int startCol, int endRow, int endCol) {
		this.startRow = startRow;
		this.startCol = startCol;
		this.endRow = endRow;
		this.endCol = endCol;
	}

	// Constructor from string.
	public LOAMove(String s) {
		s = s.toUpperCase();
		Pattern pattern = Pattern.compile("^\\s*([A-Z])\\s*([0-9]+)\\s*([A-Z])\\s*([0-9]+)\\s*$");
		Matcher matcher = pattern.matcher(s);
		if (!matcher.matches())
			throw new IllegalArgumentException("Invalid move string.");
		this.startRow = Integer.parseInt(matcher.group(2)) - 1;
		this.startCol = matcher.group(1).charAt(0) - 'A';
		this.endRow = Integer.parseInt(matcher.group(4)) - 1;
		this.endCol = matcher.group(3).charAt(0) - 'A';
	}

	// Returns the start row.
	public int getStartRow() {
		return this.startRow;
	}

	// Returns the start column.
	public int getStartCol() {
		return this.startCol;
	}

	// Returns the end row.
	public int getEndRow() {
		return this.endRow;
	}

	// Returns the end column.
	public int getEndCol() {
		return this.endCol;
	}

	// Checks whether two moves are equal.
	public boolean equals(LOAMove move) {
		if (this == move)
			return true;
		return (this.startRow == move.startRow &&
		  this.startCol == move.startCol &&
		  this.endRow == move.endRow &&
		  this.endCol == move.endCol);
	}

	// Checks whether two moves are equal.
	public boolean equals(Object o) {
		if (o instanceof LOAMove)
			return equals((LOAMove)o);
		else
			return false;
	}

	// Comparison method for the Comparable interface.
	public int compareTo(LOAMove move) {
		int result = this.startRow - move.startRow;
		if (result != 0)
			return result;
		result = this.startCol - move.startCol;
		if (result != 0)
			return result;
		result = this.endRow - move.endRow;
		if (result != 0)
			return result;
		result = this.endCol - move.endCol;
		return result;
	}

	// Encodes the move as a string.
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append((char)('A' + this.startCol));
		buffer.append(this.startRow + 1);
		buffer.append((char)('A' + this.endCol));
		buffer.append(this.endRow + 1);
		return buffer.toString();
	}
}

